// PPgDirectories.cpp : implementation file
//

#include "stdafx.h"
#include "emule.h"
#include "PPgDirectories.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

// CPPgDirectories dialog

IMPLEMENT_DYNAMIC(CPPgDirectories, CPropertyPage)
CPPgDirectories::CPPgDirectories()
	: CPropertyPage(CPPgDirectories::IDD)
	, incFiles(_T(""))
	, tempFiles(_T(""))
	, vlc(_T(""))
	, videoBackup(FALSE)
	, incomingDesc(_T(""))
	, tempDesc(_T(""))
	, vlcDesc(_T(""))
	, sharedDesc(_T(""))
	, tempDirDesc(_T(""))
{
	
}

CPPgDirectories::~CPPgDirectories()
{
}

void CPPgDirectories::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_SHARESELECTOR, m_ShareSelector);
	DDX_Control(pDX, IDC_TEMPSELECTOR, m_TempSelector);	// added by InterCeptor (multiple tempdirs) 19.11.02
	DDX_Text(pDX, IDC_INCFILES, incFiles);
	DDX_Text(pDX, IDC_TEMPFILES, tempFiles);
	DDX_Text(pDX, IDC_VLC, vlc);
	DDX_Check(pDX, IDC_VIDEOBACKUP, videoBackup);
	DDX_Control(pDX, IDC_INCFILES, incFilesEdit);
	DDX_Control(pDX, IDC_TEMPFILES, tempFilesEdit);
	DDX_Control(pDX, IDC_VLC, vlcEdit);
	DDX_Text(pDX, IDC_INCOMING_LBL, incomingDesc);
	DDX_Text(pDX, IDC_TEMP_LBL, tempDesc);
	DDX_Text(pDX, IDC_VLC_LBL, vlcDesc);
	DDX_Text(pDX, IDC_SHARED_LBL, sharedDesc);
	DDX_Text(pDX, IDC_TEMPDIR_LBL, tempDirDesc);
	DDX_Control(pDX, IDC_VIDEOBACKUP, videoBackupCheck);
	DDX_Control(pDX, IDC_SELINCDIR, selIncDirButton);
	DDX_Control(pDX, IDC_SELTEMPDIR, selTempDirButton);
	DDX_Control(pDX, IDC_SELVLC, selVlcButton);
}

BEGIN_MESSAGE_MAP(CPPgDirectories, CPropertyPage)
	ON_EN_CHANGE(IDC_INCFILES, OnEnChange)
	ON_EN_CHANGE(IDC_TEMPFILES, OnEnChange)
	ON_EN_CHANGE(IDC_VLC, OnEnChange)
	ON_BN_CLICKED(IDC_SELINCDIR, OnBnClickedSelincdir)
	ON_BN_CLICKED(IDC_SELTEMPDIR, OnBnClickedSeltempdir)
	ON_BN_CLICKED(IDC_SELVLC, OnBnClickedSelvlc)
	ON_BN_CLICKED(IDC_VIDEOBACKUP, OnEnChange)
END_MESSAGE_MAP()

BOOL CPPgDirectories::OnInitDialog()
{
	CPropertyPage::OnInitDialog();
	m_ShareSelector.Init();	
	m_TempSelector.Init();	// InterCeptor (multiple tempdirs)

	incFilesEdit.SetLimitText(MAX_PATH);
	tempFilesEdit.SetLimitText(MAX_PATH);
	vlcEdit.SetLimitText(MAX_PATH);
	
	LoadSettings();
	Localize();

	return TRUE;
}

void CPPgDirectories::LoadSettings(void)
{
	incFiles = app_prefs->GetIncomingDir();
	tempFiles = app_prefs->GetTempDir();
	vlc = app_prefs->GetVideoPlayer();
	m_ShareSelector.SetSharedDirectories(&app_prefs->shareddir_list);
	m_TempSelector.SetSharedDirectories(&app_prefs->tempdir_list);	// InterCeptor (multiple tempdirs)
	videoBackup = app_prefs->BackupPreview();
	UpdateData(FALSE);
	SetModified(FALSE);	// FoRcHa
}

bool CPPgDirectories::SelectDir(TCHAR* outdir, const CString& titletext)
{
	CoInitialize(0);	
	TCHAR buffer[MAX_PATH];
	BROWSEINFO bi = { GetSafeHwnd(), 0, buffer, titletext, BIF_VALIDATE | BIF_NEWDIALOGSTYLE | BIF_RETURNONLYFSDIRS, 0, 0, 0};
	LPITEMIDLIST pidl = SHBrowseForFolder(&bi);	
	bool done = SHGetPathFromIDList(pidl,outdir);
	LPMALLOC ppMalloc;
	if(SHGetMalloc(&ppMalloc) == NOERROR)
	{
		ppMalloc->Free(pidl);
		ppMalloc->Release();
	}
	CoUninitialize();
	return done;
}

void CPPgDirectories::OnBnClickedSelincdir()
{
	TCHAR buffer[MAX_PATH];
	if(SelectDir(buffer,GetResString(IDS_SELECT_INCOMINGDIR))) {
		incFiles = buffer;
		UpdateData(FALSE);
		SetModified();
	}
}

void CPPgDirectories::OnBnClickedSeltempdir()
{
	TCHAR buffer[MAX_PATH];
	if(SelectDir(buffer,GetResString(IDS_SELECT_TEMPDIR))) {
		tempFiles = buffer;
		UpdateData(FALSE);
		SetModified();
	}
}

void CPPgDirectories::OnBnClickedSelvlc()
{
	CFileDialog dlgFile(TRUE, _T("*.exe"), NULL,OFN_FILEMUSTEXIST | OFN_NOREADONLYRETURN ,_T("Program (*.exe)|*.exe||"));
	dlgFile.m_pOFN->lpstrInitialDir = GetPathToFile(app_prefs->GetVideoPlayer());
	
	if (dlgFile.DoModal() == IDOK) {
		vlc = dlgFile.GetPathName();
		UpdateData(FALSE);
		SetModified();
	}
}

BOOL CPPgDirectories::OnApply()
{
	if(m_bModified)
	{
		UpdateData(TRUE);
		
		if (!incFiles.IsEmpty()) {
			app_prefs->SetIncomingDir(incFiles);			
		}
		
		if (!tempFiles.IsEmpty()) {
			app_prefs->SetTempDir(tempFiles);			
		}
				
		app_prefs->SetVideoPlayer(vlc);		
		app_prefs->SetBackupPreview(videoBackup);
	
		app_prefs->shareddir_list.RemoveAll();
		m_ShareSelector.GetSharedDirectories(&app_prefs->shareddir_list);
		theApp.sharedfiles->Reload();
		
		// InterCeptor (multiple tempdirs)
		app_prefs->tempdir_list.RemoveAll();
		m_TempSelector.GetSharedDirectories(&app_prefs->tempdir_list);
	
		SetModified(FALSE);
	}
	return CPropertyPage::OnApply();
}

BOOL CPPgDirectories::OnCommand(WPARAM wParam, LPARAM lParam)
{
	if(wParam == DIRLIST_ITEMSTATECHANGED)
	{	
		if(lParam == (long)m_ShareSelector.m_hWnd)
			CheckSharedChanges();
		else if(lParam == (long)m_TempSelector.m_hWnd)
			CheckTempChanges();
	}
	return CPropertyPage::OnCommand(wParam, lParam);
}

void CPPgDirectories::Localize(void)
{
	if(::IsWindow(m_hWnd))
	{
		SetWindowText(GetResString(IDS_PW_DIR));

		incomingDesc = GetResString(IDS_PW_INCOMING);
		tempDesc = GetResString(IDS_PW_TEMP);
		vlcDesc = GetResString(IDS_PW_VLC);
		selIncDirButton.SetWindowText(GetResString(IDS_PW_BROWSE));
		selTempDirButton.SetWindowText(GetResString(IDS_PW_BROWSE));
		selVlcButton.SetWindowText(GetResString(IDS_PW_BROWSE));
		sharedDesc = GetResString(IDS_PW_SHARED);
		tempDirDesc = GetResString(IDS_PW_TEMPDIR);
		videoBackupCheck.SetWindowText(GetResString(IDS_VIDEOBACKUP));

		UpdateData(FALSE);
	}
}

void CPPgDirectories::CheckSharedChanges(void)
{
	CStringList lstShared;
	m_ShareSelector.GetSharedDirectories(&lstShared);
	
	if(lstShared.GetCount() == app_prefs->shareddir_list.GetCount())
	{	
		POSITION pos1 = lstShared.GetHeadPosition();
		POSITION pos2 = app_prefs->shareddir_list.GetHeadPosition();
		for(int i = 0; i < lstShared.GetCount(); i++)
		{	
			if(lstShared.GetNext(pos1) != app_prefs->shareddir_list.GetNext(pos2))
			{
				SetModified();
				i = lstShared.GetCount();
			}
		}
	}			
	else
		SetModified();
}

void CPPgDirectories::CheckTempChanges(void)
{
	CStringList lstTemp;
	m_TempSelector.GetSharedDirectories(&lstTemp);
	
	if(lstTemp.GetCount() == app_prefs->tempdir_list.GetCount())
	{	
		POSITION pos1 = lstTemp.GetHeadPosition();
		POSITION pos2 = app_prefs->tempdir_list.GetHeadPosition();
		for(int i = 0; i < lstTemp.GetCount(); i++)
		{	
			if(lstTemp.GetNext(pos1) != app_prefs->tempdir_list.GetNext(pos2))
			{
				SetModified();
				i = lstTemp.GetCount();
			}
		}
	}			
	else
		SetModified();
}
