
#include "stdafx.h"
#include <assert.h>
#include "mdump.h"
#include "otherfunctions.h"
#include "emule.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

static LPCTSTR m_szEmpty = _T("");

LPCTSTR	MiniDumper::m_szAppName = NULL;
LPCTSTR	MiniDumper::m_szCrashMsgForWin9x = m_szEmpty;
LPCTSTR	MiniDumper::m_szKeepDumpFile = m_szEmpty;
LPCTSTR	MiniDumper::m_szCantUseDBGHelp = m_szEmpty;
LPCTSTR	MiniDumper::m_szDownloadDBGHelp = m_szEmpty;
LPCTSTR	MiniDumper::m_szSavedDump = m_szEmpty;
LPCTSTR	MiniDumper::m_szFailedToSave = m_szEmpty;
bool	MiniDumper::m_bWritten;

MiniDumper::MiniDumper( LPCTSTR szAppName )
{
	// if this assert fires then you have two instances of MiniDumper
	// which is not allowed
	assert(m_szAppName==NULL);

	m_bWritten = false;
	m_szAppName = _tcsdup(szAppName ? szAppName : _T("Application"));

	::SetUnhandledExceptionFilter(TopLevelFilter);
}

void MiniDumper::LoadStrings()
{
	m_szCrashMsgForWin9x = _tcsdup(GetResString(IDS_DUMP_CRASHWIN9X));
	m_szKeepDumpFile = _tcsdup(GetResString(IDS_DUMP_KEEPDUMPFILE));
	m_szCantUseDBGHelp = _tcsdup(GetResString(IDS_DUMP_CANTUSEDBGHELP));
	m_szDownloadDBGHelp = _tcsdup(GetResString(IDS_DUMP_DOWNLOADDBG));
	m_szSavedDump = _tcsdup(GetResString(IDS_DUMP_SAVEDDUMP));
	m_szFailedToSave = _tcsdup(GetResString(IDS_DUMP_FAILEDTOSAVE));
}

MiniDumper::~MiniDumper()
{
	if(m_szAppName)
		free((void*)m_szAppName);
	if(m_szCrashMsgForWin9x != m_szEmpty)
		free((void*)m_szCrashMsgForWin9x);
	if(m_szKeepDumpFile != m_szEmpty)
		free((void*)m_szKeepDumpFile);
	if(m_szCantUseDBGHelp != m_szEmpty)
		free((void*)m_szCantUseDBGHelp);
	if(m_szDownloadDBGHelp != m_szEmpty)
		free((void*)m_szDownloadDBGHelp);
	if(m_szSavedDump != m_szEmpty)
		free((void*)m_szSavedDump);
	if(m_szFailedToSave != m_szEmpty)
		free((void*)m_szFailedToSave);
}

void CloseAfterTimeout(void*)
{
	Sleep(60 * 1000);	// wait minute and terminate
	TerminateProcess(GetCurrentProcess(), 0);
}

LONG MiniDumper::TopLevelFilter( struct _EXCEPTION_POINTERS *pExceptionInfo )
{
    theApp.m_app_state = theApp.APP_STATE_SHUTINGDOWN;

	// Start timeout thread
	_beginthread(CloseAfterTimeout, 0, NULL);

	// If already written, wait for process to terminate
	if(m_bWritten)
	{
		Sleep(60 * 1000);
		return EXCEPTION_CONTINUE_EXECUTION;
	}

	BOOL bIsNTBased	=	FALSE;
	// Can't save dump file for Win9x
	WORD wWinVer = DetectWinVersion(bIsNTBased);
	if(!bIsNTBased)
	{
		MessageBox(NULL, m_szCrashMsgForWin9x, m_szAppName, MB_OK | MB_ICONSTOP);
		ExitProcess(0);
		return EXCEPTION_EXECUTE_HANDLER;
	}

	LONG retval = EXCEPTION_CONTINUE_SEARCH;

	// firstly see if dbghelp.dll is around and has the function we need
	// look next to the EXE first, as the one in System32 might be old 
	// (e.g. Windows 2000)
	HMODULE hDll = NULL;
	TCHAR szDbgHelpPath[_MAX_PATH];

	if (GetModuleFileName( NULL, szDbgHelpPath, _MAX_PATH ))
	{
		TCHAR *pSlash = _tcsrchr( szDbgHelpPath, '\\' );
		if (pSlash)
		{
			_tcscpy( pSlash+1, _T("DBGHELP.DLL") );
			hDll = ::LoadLibrary( szDbgHelpPath );
		}
	}

	if (hDll==NULL)
	{
		// load any version we can
		hDll = LoadLibrary( _T("DBGHELP.DLL") );
	}

	LPCTSTR szResult = NULL;

	if (hDll)
	{
		MINIDUMPWRITEDUMP pDump = (MINIDUMPWRITEDUMP)GetProcAddress( hDll, "MiniDumpWriteDump" );
		if (pDump)
		{
			TCHAR szDumpPath[_MAX_PATH];
			TCHAR szScratch [0x500];

			// work out a good place for the dump file
			GetModuleFileName(0,szDumpPath, 490);
			LPTSTR pszFileName = _tcsrchr(szDumpPath, '\\') + 1;
			*pszFileName = '\0';

			_tcscat( szDumpPath, m_szAppName );
			_tcscat( szDumpPath, _T(".dmp") );
			DeleteFile(szDumpPath);

			// save dump
			// create the file
			HANDLE hFile = CreateFile( szDumpPath, GENERIC_WRITE, FILE_SHARE_WRITE, NULL, CREATE_ALWAYS,
										FILE_ATTRIBUTE_NORMAL, NULL );

			int nStatus = 0;
			if (hFile!=INVALID_HANDLE_VALUE)
			{
				_MINIDUMP_EXCEPTION_INFORMATION ExInfo;

				ExInfo.ThreadId = GetCurrentThreadId();
				ExInfo.ExceptionPointers = pExceptionInfo;
				ExInfo.ClientPointers = NULL;

				// write the dump
				BOOL bOK = pDump( GetCurrentProcess(), GetCurrentProcessId(), hFile, MiniDumpNormal, &ExInfo, NULL, NULL );
				CloseHandle(hFile);
				if (bOK)
					nStatus = 1;
				else
					nStatus = 2;
			}
			else
				nStatus = 3;

			m_bWritten = true;
			// ask the user if they want to keep a dump file
			if (MessageBox( NULL, m_szKeepDumpFile, m_szAppName, MB_YESNO | MB_ICONSTOP ) == IDYES)
			{
				switch(nStatus)
				{
				case 1:
					// Please MODDERS change email in string. THX!
					_stprintf( szScratch, m_szSavedDump, szDumpPath );
					szResult = szScratch;
					retval = EXCEPTION_EXECUTE_HANDLER;
					break;
				case 2:
				case 3:
					_stprintf( szScratch, m_szFailedToSave, szDumpPath, GetLastError() );
					szResult = szScratch;
					break;
				}
			}
			else
				DeleteFile(szDumpPath);
		}
		else
			szResult = m_szCantUseDBGHelp;
	}
	else
		szResult = m_szDownloadDBGHelp;

	if (szResult)
		MessageBox( NULL, szResult, m_szAppName, MB_OK | MB_ICONSTOP );
	ExitProcess(0);
	return retval;
}
